/*
** File: adpexrc6t.h  
** Project: ADB3 core driver
** Purpose: Definitions specific to ADPE-XRC-6T(L).
**
** (C) Copyright Alpha Data 2011
*/

#ifndef _ADATA_CORE_ADPEXRC6T_H
#define _ADATA_CORE_ADPEXRC6T_H

#include <df.h>
#include "adb3.h"
#include "admxrc6t1.h"
#include "admxrc6tx.h"

#define ADPEXRC6T_NUM_DMA_CHANNEL  (4)
#define ADPEXRC6TL_NUM_DMA_CHANNEL (2)

/* Always LITTLE ENDIAN when stored in hardware */
#pragma pack(1)
typedef struct _VpdAdpexrc6tRev0 {
	uint16_t version;       /* Revision of this structure (0) */
	uint16_t length;        /* Length of this structure including checksum */
	uint32_t serial;
	uint32_t modifications;
  uint8_t  tempGrade;
  uint8_t  _reserved1;
	uint8_t  pcbRev;
  uint8_t  _reserved2;
	uint32_t si5338Ref;     /* Frequency of SI5338 reference oscillator, in Hz */
	uint32_t idelayRef;     /* IDELAY reference clock frequency, in Hz */
	uint32_t gtpRef;        /* GTP/GTX reference clock frequency, in Hz */
  uint32_t _reserved3;
	struct {
		uint16_t speed;       /* In units of 100 kHz */
		uint8_t	 order;       /* Log base 2 of size in words */
    uint8_t  _reserved4;
    uint16_t link;        /* Byte offset in VPD of extended info */
    uint16_t _reserved5;
	} sdram[2];
  struct {
    uint8_t  stepping[8];
    uint8_t  speedGrade[4];
    uint16_t device;
    uint16_t _reserved6;
  } fpga;
  uint8_t  checksum;      /* This MUST ALWAYS be the last member in the struct */
} VpdAdpexrc6tRev0;
#pragma pack()

/* Values for VpdAdpexrc6tRev0::modifications */

/* Values for VpdAdpexrc6tRev0::fpga::tempGrade */
#define ADPEXRC6T_VPD_TEMPGRADE_COMMERCIAL  ADMXRC6TL_VPD_TEMPGRADE_COMMERCIAL
#define ADPEXRC6T_VPD_TEMPGRADE_INDUSTRIAL  ADMXRC6TL_VPD_TEMPGRADE_INDUSTRIAL
#define ADPEXRC6T_VPD_TEMPGRADE_EXTENDED    ADMXRC6TL_VPD_TEMPGRADE_EXTENDED

/* Values for ADMXRC6Tx_V_I2CCTL_I2CADDR() */
#define ADPEXRC6T_I2C_SLOT_SI5338           (0x70U)

/* Fields for ModelRegsAdmxrc6tx::intCtl */
#define ADPEXRC6T_V_INTCTL_ADVRXNE    DF_VAL32S(0x1U, 5, 1)  /* AVR uC RX FIFO not empty interrupt enable */
#define ADPEXRC6T_V_INTCTL_ADVTXE     DF_VAL32S(0x1U, 6, 1)  /* AVR uC TX FIFO empty interrupt enable */
#define ADPEXRC6T_V_INTCTL_LS_ADVRXNE DF_VAL32S(0x1U, 13, 1) /* 1 => AVR uC RX FIFO not empty interrupt is level-sensitive */
#define ADPEXRC6T_V_INTCTL_LS_ADVTXE  DF_VAL32S(0x1U, 14, 1) /* 1 => AVR uC TX FIFO empty interrupt is level-sensitive */

#define ADPEXRC6T_V_INTCTL_LEVELBITS (ADMXRC6Tx_V_INTCTL_SYSLEVEL | ADPEXRC6T_V_INTCTL_LS_ADVRXNE | ADPEXRC6T_V_INTCTL_LS_ADVTXE)

/* Fields for ModelRegsAdmxrc6tx::intStat */
#define ADPEXRC6T_V_INTSTAT_ADVRXNE   DF_VAL32S(0x1U, 5, 1)  /* AVR uC RX FIFO not empty interrupt active */
#define ADPEXRC6T_V_INTSTAT_ADVTXE    DF_VAL32S(0x1U, 6, 1)  /* AVR uC TX FIFO empty interrupt active */

#define ADPEXRC6T_AVR_FLASH_PAGE_SIZE   (0x100U) /* Size in bytes of a Flash memory page in AVR uC */
#define ADPEXRC6T_AVR_FLASH_SIZE       (0x4000U) /* Size in bytes of Flash memory in AVR uC */
#define ADPEXRC6T_AVR_FLASH_VPD_OFFSET (0x3F00U) /* Offset (bytes) into of Flash memory in AVR uC where VPD is located */

#endif
