/*
** File: sempool.c  
** Project: VxWorks Driver Framework
** Purpose: Semaphore pool
**
** (C) Copyright Alpha Data 2010
*/

#include "df.h"
#include "dfvxworks.h"

#if DFVXW_EVENT_USE_SEMPOOL

# define DFVXW_SEMBPOOL_INITIAL_SIZE (0x100)

SEM_ID g_dfVxwSemBPoolArray[DFVXW_SEMBPOOL_INITIAL_SIZE];
unsigned int g_dfVxwSemBPoolPtr = 0;
unsigned int g_dfVxwSemBPoolSize = 0;
DfSpinLock g_dfVxwSemBPoolLock;

# define DFVXW_SEMMPOOL_INITIAL_SIZE (0x100)

SEM_ID g_dfVxwSemMPoolArray[DFVXW_SEMMPOOL_INITIAL_SIZE];
unsigned int g_dfVxwSemMPoolPtr = 0;
unsigned int g_dfVxwSemMPoolSize = 0;
DfSpinLock g_dfVxwSemMPoolLock;

#endif

boolean_t
dfVxwSemPoolInit(
  void)
{
#if DFVXW_EVENT_USE_SEMPOOL
  unsigned int i;

  g_dfVxwSemBPoolSize = DFVXW_SEMBPOOL_INITIAL_SIZE;
  for (i = 0; i < g_dfVxwSemBPoolSize; i++) {
    g_dfVxwSemBPoolArray[i] = NULL;
  }
  g_dfVxwSemBPoolPtr = 0;
  dfSpinLockInit(&g_dfVxwSemBPoolLock);

  g_dfVxwSemMPoolSize = DFVXW_SEMMPOOL_INITIAL_SIZE;
  for (i = 0; i < g_dfVxwSemMPoolSize; i++) {
    g_dfVxwSemMPoolArray[i] = NULL;
  }
  g_dfVxwSemMPoolPtr = 0;
  dfSpinLockInit(&g_dfVxwSemMPoolLock);

  for (i = 0; i < g_dfVxwSemBPoolSize; i++) {
    g_dfVxwSemBPoolArray[i] = semBCreate(SEM_Q_FIFO, SEM_EMPTY);
    if (g_dfVxwSemBPoolArray[i] == NULL) {
      dfDebugPrint(0, ("*** dfVxwSemPoolInit: failed to create binary semaphore %u, errno=%d\n", i, errno));
      dfVxwSemPoolUninit();
      return FALSE;
    }
  }

  for (i = 0; i < g_dfVxwSemMPoolSize; i++) {
    g_dfVxwSemMPoolArray[i] = semMCreate(SEM_Q_FIFO);
    if (g_dfVxwSemMPoolArray[i] == NULL) {
      dfDebugPrint(0, ("*** dfVxwSemPoolInit: failed to create mutex semaphore %u, errno=%d\n", i, errno));
      dfVxwSemPoolUninit();
      return FALSE;
    }
  }
#endif

  return TRUE;
}

void
dfVxwSemPoolUninit(
  void)
{
#if DFVXW_EVENT_USE_SEMPOOL
  unsigned int i;

  for (i = 0; i < g_dfVxwSemMPoolSize; i++) {
    if (g_dfVxwSemMPoolArray[i] != NULL) {
      semDelete(g_dfVxwSemMPoolArray[i]);
      g_dfVxwSemMPoolArray[i] = NULL;
    }
  }
  if (g_dfVxwSemMPoolPtr) {
    dfDebugPrint(0, ("*** dfVxwSemPoolUninit: some mutex semaphores not returned to pool, semPoolPtr=%d\n", g_dfVxwSemMPoolPtr));
  }
  g_dfVxwSemMPoolPtr = 0;

  for (i = 0; i < g_dfVxwSemBPoolSize; i++) {
    if (g_dfVxwSemBPoolArray[i] != NULL) {
      semDelete(g_dfVxwSemBPoolArray[i]);
      g_dfVxwSemBPoolArray[i] = NULL;
    }
  }
  if (g_dfVxwSemBPoolPtr) {
    dfDebugPrint(0, ("*** dfVxwSemPoolUninit: some binary semaphores not returned to pool, semPoolPtr=%d\n", g_dfVxwSemBPoolPtr));
  }
  g_dfVxwSemBPoolPtr = 0;
#endif
}
